from __future__ import absolute_import, unicode_literals

from functools import reduce
import re

from django.db.models import Q

from towel import queryset_transform


def normalize_query(query_string,
                    findterms=re.compile(r'"([^"]+)"|(\S+)').findall,
                    normspace=re.compile(r'\s{2,}').sub):
    """
    Splits the query string in invidual keywords, getting rid of unecessary
    spaces and grouping quoted words together.

    Example::

        >>> normalize_query(' some random  words "with   quotes  " and spaces')
        ['some', 'random', 'words', 'with quotes', 'and', 'spaces']

    """
    return [normspace(' ', (t[0] or t[1]).strip())
            for t in findterms(query_string)]


class SearchManager(queryset_transform.TransformManager):
    """
    Stupid searching manager

    Does not use fulltext searching abilities of databases. Constructs a query
    searching specified fields for a freely definable search string. The
    individual terms may be grouped by using apostrophes, and can be prefixed
    with + or - signs to specify different searching modes::

        +django "shop software" -satchmo

    Usage example::

        class MyModelManager(SearchManager):
            search_fields = ('field1', 'name', 'related__field')

        class MyModel(models.Model):
            # ...

            objects = MyModelManager()

        MyModel.objects.search('yeah -no')
    """

    search_fields = ()

    def search(self, query):
        """
        This implementation stupidly forwards to _search, which does the
        gruntwork.

        Put your customizations in here.
        """

        return self._search(query)

    def _search(self, query, fields=None, queryset=None):
        if queryset is None:
            queryset = self.all()

        if fields is None:
            fields = self.search_fields

        if not query or not fields:
            return queryset

        for keyword in normalize_query(query):
            negate = False
            if len(keyword) > 1:
                if keyword[0] == '-':
                    keyword = keyword[1:]
                    negate = True
                elif keyword[0] == '+':
                    keyword = keyword[1:]

            if negate:
                q = reduce(
                    lambda p, q: p & q,
                    (~Q(**{'%s__icontains' % f: keyword}) for f in fields),
                    Q())
            else:
                q = reduce(
                    lambda p, q: p | q,
                    (Q(**{'%s__icontains' % f: keyword}) for f in fields),
                    Q())

            queryset = queryset.filter(q)

        return queryset



"""
Forms
=====

These three form subclasses will automatically add limitation by tenant
to all form fields with a ``queryset`` attribute.

.. warning::

    If you customized the dropdown using ``choices`` you have to limit the
    choices by the current tenant yourself.
"""

from __future__ import absolute_import, unicode_literals

from django import forms
from django.db.models import FieldDoesNotExist

from towel import forms as towel_forms
from towel.mt import client_model
from towel.utils import safe_queryset_and


def _process_fields(form, request):
    for field in form.fields.values():
        if hasattr(field, 'queryset'):
            model = field.queryset.model

            field.queryset = safe_queryset_and(
                field.queryset,
                model.objects.for_access(request.access),
            )


class Form(forms.Form):
    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request')
        super(Form, self).__init__(*args, **kwargs)
        _process_fields(self, self.request)


class ModelForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request')
        super(ModelForm, self).__init__(*args, **kwargs)
        _process_fields(self, self.request)

    def save(self, commit=True):
        Client = client_model()
        attr = Client.__name__.lower()
        try:
            field = self.instance._meta.get_field(attr)
        except FieldDoesNotExist:
            field = None
        if (field and field.rel and field.rel.to
                and issubclass(field.rel.to, Client)):
            setattr(self.instance, attr, getattr(self.request.access, attr))

        return super(ModelForm, self).save(commit=commit)


class SearchForm(towel_forms.SearchForm):
    def post_init(self, request):
        self.request = request
        _process_fields(self, self.request)


class BatchForm(towel_forms.BatchForm):
    def __init__(self, *args, **kwargs):
        super(BatchForm, self).__init__(*args, **kwargs)
        _process_fields(self, self.request)



from __future__ import absolute_import, unicode_literals

from django.core.urlresolvers import NoReverseMatch, reverse
from django.utils.six.moves import http_client

from towel.utils import app_model_label


class APIException(Exception):
    """
    Custom exception which signals a problem detected somewhere inside
    the API machinery.

    Usage::

        # Use official W3C error names from ``httplib.responses``
        raise ClientError(status=httplib.NOT_ACCEPTABLE)

    or::

        raise ServerError('Not implemented, go away',
            status=httplib.NOT_IMPLEMENTED)

    Additional information can be passed through by setting the ``data``
    argument to a dict instance. The :py:exc:`~towel.api.APIException` handler
    will merge the dict into the default error data and return everything
    to the client::

        raise APIException('Validation failed', data={
            'form': form.errors,
            })
    """

    #: The default response is '400 Bad request'
    default_status = http_client.BAD_REQUEST

    def __init__(self, error_message=None, status=None, data={}):
        super(Exception, self).__init__(error_message)

        self.status = self.default_status if status is None else status
        if error_message is None:
            self.error_message = http_client.responses.get(self.status, '')
        else:
            self.error_message = error_message

        self.data = data


def api_reverse(model, ident, api_name='api', fail_silently=False, **kwargs):
    """
    Determines the canonical URL of API endpoints for arbitrary models

    ``model`` is the Django model you want to use, ident should be one of
    ``list``, ``set`` or ``detail`` at the moment, additional keyword arguments
    are forwarded to the ``django.core.urlresolvers.reverse`` call.

    Usage::

        api_reverse(Product, 'detail', pk=42)

    Passing an instance works too::

        api_reverse(instance, 'detail', pk=instance.pk)
    """
    try:
        return reverse(
            '_'.join(
                (api_name,) + app_model_label(model) + (ident,)),
            kwargs=kwargs)
    except NoReverseMatch:
        if fail_silently:
            return None
        raise



from __future__ import absolute_import, unicode_literals

import re

from django import template
from django.utils.html import conditional_escape
from django.utils.safestring import mark_safe

from towel.utils import parse_args_and_kwargs, resolve_args_and_kwargs


register = template.Library()


def flatatt(attrs):
    """
    Convert a dictionary of attributes to a single string.
    The returned string will contain a leading space followed by key="value",
    XML-style pairs.  It is assumed that the keys do not need to be
    XML-escaped.  If the passed dictionary is empty, then return an empty
    string.
    """
    return ''.join([
        ' %s="%s"' % (k, conditional_escape(v))
        for k, v in attrs.items()])


@register.tag
def region(parser, token):
    """
    Defines a live-updateable region::

        {% region "identifier" fields="family_name,given_name" tag="div" %}
            {# Template code #}
        {% endregion %}

    The identifier should be a short string which is unique for the whole
    project, or at least for a given view. It is used to identify the region
    when live-updating, it should therefore be usable as a part of a HTML
    ``id`` attribute. The identifer should not start with an underscore,
    those names are reserved for internal bookkeeping.

    ``fields`` is a comma-separated list of fields (or other identifiers)
    which are used inside the given region. It is recommended to use the
    field and relation names here, but you are free to use anything you
    want. It can also be left empty if you purely want to update regions by
    their identifier.

    The ``tag`` argument defines the HTML tag used to render the region.
    The default tag is a ``div``.

    Additional keyword arguments will be rendered as attributes. This can
    be used to specify classes, data attributes or whatever you desire.
    """

    nodelist = parser.parse(('endregion',))
    parser.delete_first_token()

    return RegionNode(
        nodelist, *parse_args_and_kwargs(parser, token.split_contents()[1:]))


class RegionNode(template.Node):
    def __init__(self, nodelist, args, kwargs):
        self.nodelist = nodelist
        self.args = args
        self.kwargs = kwargs

    def render(self, context):
        args, kwargs = resolve_args_and_kwargs(context, self.args, self.kwargs)
        return self._render(context, *args, **kwargs)

    def _render(self, context, identifier, fields='', tag='div', **kwargs):
        regions = context.get('regions')

        region_id = 'twrg-%s' % identifier
        output = self.nodelist.render(context)

        if regions is not None:
            regions[region_id] = output
            dependencies = regions.setdefault('_dependencies', {})

            for field in re.split('[,\s]+', str(fields)):
                dependencies.setdefault(field, []).append(region_id)

        kwargs['id'] = region_id

        return mark_safe('<{tag} {attrs}>{output}</{tag}>'.format(
            attrs=flatatt(kwargs),
            output=output,
            tag=tag,
        ))



from __future__ import absolute_import, unicode_literals

from django.contrib.auth.backends import ModelBackend as _ModelBackend
from django.contrib.auth.models import User


class ModelBackend(_ModelBackend):
    """
    Add the following to your ``settings.py`` to be able to login with
    email addresses too::

        AUTHENTICATION_BACKENDS = (
            'towel.auth.ModelBackend',
        )
    """
    def authenticate(self, username=None, password=None):
        try:
            user = User.objects.get(username=username)
        except User.DoesNotExist:
            try:
                user = User.objects.get(email=username)
            except User.DoesNotExist:
                return None

        if user.check_password(password):
            return user

        return None



"""
Models for multitenant Django projects
======================================

The models for ``towel.mt`` have to be provided by the project where
``towel.mt`` is used, that's why this file is empty.

The simplest models might look like that::

    from django.contrib.auth.models import User
    from django.db import models


    class Client(models.Model):
        name = models.CharField(max_length=100)


    class Access(models.Model):
        EMPLOYEE = 10
        MANAGEMENT = 20

        ACCESS_CHOICES = (
            (EMPLOYEE, 'employee'),
            (MANAGEMENT, 'management'),
            )

        client = models.ForeignKey(Client)
        user = models.OneToOneField(User)
        access = models.SmallIntegerField(choices=ACCESS_CHOICES)


API methods can be protected as follows::

    from towel.api import API
    from towel.api.decorators import http_basic_auth
    from towel.mt.api import Resource, api_access

    # Require a valid login and an associated Access model:
    api_v1 = API('v1', decorators=[
        csrf_exempt,
        http_basic_auth,
        api_access(Access.EMPLOYEE),
        ])
    api_v1.register(SomeModel,
        view_class=Resource,
        )


Other views::

    from towel.mt import AccessDecorator

    # Do this once somewhere in your project
    access = AccessDecorator()


    @access(Access.MANAGEMENT)
    def management_only_view(request):
        # ...
"""
# Intentionally left empty.

from __future__ import absolute_import, unicode_literals



"""
Middleware for a lazy ``request.access`` attribute
==================================================
"""

from __future__ import absolute_import, unicode_literals

from django.db.models import ObjectDoesNotExist
from django.utils.functional import SimpleLazyObject


def get_access(request):
    try:
        return request.user.access
    except (AttributeError, ObjectDoesNotExist):
        return None


class LazyAccessMiddleware(object):
    """
    This middleware (or something equivalent providing a ``request.access``
    attribute must be put in ``MIDDLEWARE_CLASSES`` to use the helpers in
    ``towel.mt``.
    """
    def process_request(self, request):
        request.access = SimpleLazyObject(lambda: get_access(request))



"""MIME-Type Parser

This module provides basic functions for handling mime-types. It can handle
matching mime-types against a list of media-ranges. See section 14.1 of the
HTTP specification [RFC 2616] for a complete explanation.

   http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.1

Contents:
 - parse_mime_type():   Parses a mime-type into its component parts.
 - parse_media_range(): Media-ranges are mime-types with wild-cards and a 'q'
                          quality parameter.
 - quality():           Determines the quality ('q') of a mime-type when
                          compared against a list of media-ranges.
 - quality_parsed():    Just like quality() except the second parameter must be
                          pre-parsed.
 - best_match():        Choose the mime-type with the highest quality ('q')
                          from a list of candidates.
"""
from functools import reduce

__version__ = '0.1.4.1'
__author__ = 'Joe Gregorio'
__email__ = 'joe@bitworking.org'
__license__ = 'MIT License'
__credits__ = ''


def parse_mime_type(mime_type):
    """Parses a mime-type into its component parts.

    Carves up a mime-type and returns a tuple of the (type, subtype, params)
    where 'params' is a dictionary of all the parameters for the media range.
    For example, the media range 'application/xhtml;q=0.5' would get parsed
    into:

       ('application', 'xhtml', {'q', '0.5'})
    """
    parts = mime_type.split(';')
    params = dict([tuple([s.strip() for s in param.split('=', 1)])
                  for param in parts[1:]
                   ])
    full_type = parts[0].strip()
    # Java URLConnection class sends an Accept header that includes a
    # single '*'. Turn it into a legal wildcard.
    if full_type == '*':
        full_type = '*/*'
    (type, subtype) = full_type.split('/')

    return (type.strip(), subtype.strip(), params)


def parse_media_range(range):
    """Parse a media-range into its component parts.

    Carves up a media range and returns a tuple of the (type, subtype,
    params) where 'params' is a dictionary of all the parameters for the media
    range.  For example, the media range 'application/*;q=0.5' would get parsed
    into:

       ('application', '*', {'q', '0.5'})

    In addition this function also guarantees that there is a value for 'q'
    in the params dictionary, filling it in with a proper default if
    necessary.
    """
    (type, subtype, params) = parse_mime_type(range)
    if not 'q' in params or not params['q'] or \
            float(params['q']) > 1 or float(params['q']) < 0:
        params['q'] = '1'

    return (type, subtype, params)


def quality_and_fitness_parsed(mime_type, parsed_ranges):
    """Find the best match for a mime-type amongst parsed media-ranges.

    Find the best match for a given mime-type against a list of media_ranges
    that have already been parsed by parse_media_range(). Returns a tuple of
    the fitness value and the value of the 'q' quality parameter of the best
    match, or (-1, 0) if no match was found. Just as for quality_parsed(),
    'parsed_ranges' must be a list of parsed media ranges.
    """
    best_fitness = -1
    best_fit_q = 0
    (target_type, target_subtype, target_params) =\
        parse_media_range(mime_type)
    for (type, subtype, params) in parsed_ranges:
        type_match = (type == target_type or
                      type == '*' or
                      target_type == '*')
        subtype_match = (subtype == target_subtype or
                         subtype == '*' or
                         target_subtype == '*')
        if type_match and subtype_match:
            param_matches = reduce(lambda x, y: x + y, [1 for (key, value) in
                                   target_params.items() if key != 'q' and
                                   key in params and value == params[key]], 0)
            fitness = (type == target_type) and 100 or 0
            fitness += (subtype == target_subtype) and 10 or 0
            fitness += param_matches
            if fitness > best_fitness:
                best_fitness = fitness
                best_fit_q = params['q']

    return float(best_fit_q), best_fitness


def quality_parsed(mime_type, parsed_ranges):
    """Find the best match for a mime-type amongst parsed media-ranges.

    Find the best match for a given mime-type against a list of media_ranges
    that have already been parsed by parse_media_range(). Returns the 'q'
    quality parameter of the best match, 0 if no match was found. This function
    bahaves the same as quality() except that 'parsed_ranges' must be a list of
    parsed media ranges. """

    return quality_and_fitness_parsed(mime_type, parsed_ranges)[0]


def quality(mime_type, ranges):
    """Return the quality ('q') of a mime-type against a list of media-ranges.

    Returns the quality 'q' of a mime-type when compared against the
    media-ranges in ranges. For example:

    >>> quality('text/html','text/*;q=0.3, text/html;q=0.7,
                  text/html;level=1, text/html;level=2;q=0.4, */*;q=0.5')
    0.7

    """
    parsed_ranges = [parse_media_range(r) for r in ranges.split(',')]

    return quality_parsed(mime_type, parsed_ranges)


def best_match(supported, header):
    """Return mime-type with the highest quality ('q') from list of candidates.

    Takes a list of supported mime-types and finds the best match for all the
    media-ranges listed in header. The value of header must be a string that
    conforms to the format of the HTTP Accept: header. The value of 'supported'
    is a list of mime-types. The list of supported mime-types should be sorted
    in order of increasing desirability, in case of a situation where there is
    a tie.

    >>> best_match(['application/xbel+xml', 'text/xml'],
                   'text/*;q=0.5,*/*; q=0.1')
    'text/xml'
    """
    split_header = _filter_blank(header.split(','))
    parsed_header = [parse_media_range(r) for r in split_header]
    weighted_matches = []
    pos = 0
    for mime_type in supported:
        weighted_matches.append((quality_and_fitness_parsed(mime_type,
                                 parsed_header), pos, mime_type))
        pos += 1
    weighted_matches.sort()

    return weighted_matches[-1][0][0] and weighted_matches[-1][2] or ''


def _filter_blank(i):
    for s in i:
        if s.strip():
            yield s

# flake8: noqa



"""
This is mostly equivalent with Django's inline formsets mechanism, but
used together with editlive.
"""

from __future__ import absolute_import, unicode_literals

import json

from django.core.exceptions import PermissionDenied
from django.forms.models import model_to_dict
from django.http import HttpResponse
from django.shortcuts import get_object_or_404, redirect

from towel.resources.base import DetailView, FormView, LiveFormView, DeleteView
from towel.utils import changed_regions


class ChildMixin(object):
    base_template = 'modal.html'
    parent_attr = 'parent'

    def get_parent_class(self):
        return self.model._meta.get_field(self.parent_attr).rel.to

    def get_parent_queryset(self):
        return self.get_parent_class()._default_manager.all()

    def get_parent(self):
        return get_object_or_404(
            self.get_parent_queryset(),
            pk=self.kwargs[self.parent_attr])

    def update_parent(self):
        regions = DetailView.render_regions(
            self,
            model=self.parent.__class__,
            object=self.parent)

        return HttpResponse(
            json.dumps(changed_regions(regions, [
                '%s_set' % self.model.__name__.lower(),
            ])),
            content_type='application/json')


class ChildFormView(ChildMixin, FormView):
    def get_form_kwargs(self, **kwargs):
        kwargs['prefix'] = self.model.__name__.lower()
        return super(ChildMixin, self).get_form_kwargs(**kwargs)

    def form_valid(self, form):
        setattr(form.instance, self.parent_attr, self.parent)
        self.object = form.save()
        return self.update_parent()


class ChildAddView(ChildFormView):
    def get(self, request, *args, **kwargs):
        if not self.allow_add(silent=False):
            return redirect(self.url('list'))
        self.parent = self.get_parent()
        form = self.get_form()
        return self.render_to_response(self.get_context_data(form=form))

    def post(self, request, *args, **kwargs):
        if not self.allow_add(silent=False):
            return redirect(self.url('list'))
        self.parent = self.get_parent()
        form = self.get_form()
        if form.is_valid():
            return self.form_valid(form)
        return self.form_invalid(form)


class ChildEditView(ChildFormView):
    def get(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.parent = getattr(self.object, self.parent_attr)
        if not self.allow_edit(self.object, silent=False):
            return redirect(self.object)
        form = self.get_form()
        context = self.get_context_data(form=form, object=self.object)
        return self.render_to_response(context)

    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.parent = getattr(self.object, self.parent_attr)
        if not self.allow_edit(self.object, silent=False):
            return redirect(self.object)
        form = self.get_form()
        if form.is_valid():
            return self.form_valid(form)
        return self.form_invalid(form)


class LiveChildFormView(ChildMixin, LiveFormView):
    def post(self, request, *args, **kwargs):
        self.object = self.get_object()
        self.parent = getattr(self.object, self.parent_attr)
        if not self.allow_edit(self.object, silent=True):
            raise PermissionDenied

        form_class = self.get_form_class()
        data = model_to_dict(
            self.object,
            fields=form_class._meta.fields,
            exclude=form_class._meta.exclude)

        for key, value in request.POST.items():
            data[key] = value

        form = form_class(**self.get_form_kwargs(data=data))

        if form.is_valid():
            return self.form_valid(form)

        # TODO that's actually quite ugly
        return HttpResponse('%s' % form.errors)

    def form_valid(self, form):
        self.object = form.save()
        return self.update_parent()


class ChildDeleteView(ChildMixin, DeleteView):
    def deletion_form_valid(self, form):
        """
        On successful form validation, the object is deleted and the user is
        redirected to the list view of the model.
        """
        self.parent = self.get_parent()
        self.object.delete()
        return self.update_parent()



# flake8: noqa
from .api import API, serialize_model_instance
from .base import APIException, api_reverse
from .resources import Resource
from .serializers import Serializer

# Is that really public API?
from .parsers import RequestParser
from .utils import querystring



from __future__ import absolute_import, unicode_literals

from collections import namedtuple
import logging

from django.http import Http404, HttpResponse
from django.shortcuts import get_object_or_404
from django.utils.six.moves import http_client
from django.views import generic

from .base import APIException, api_reverse
from .parsers import RequestParser
from .serializers import Serializer
from .utils import querystring


#: The ``page`` object from ``Resource.objects``
Page = namedtuple('Page', 'queryset offset limit full_queryset')

logger = logging.getLogger('towel.api')


class Resource(generic.View):
    """
    Resource for exposing Django models somewhat RESTy
    """

    #: The API to which this resource is bound to.
    api = None

    #: The model exposed by this resource.
    model = None

    #: Prefiltered queryset for this resource. Defaults to
    #: ``model._default_manager.all()`` if unset.
    queryset = None

    #: Default instance count for list views
    limit_per_page = 20
    #: Higher values than this will not be accepted for ``limit``
    max_limit_per_page = 1000

    #: Almost the same as ``django.views.generic.View.http_method_names`` but
    #: not quite, we allow ``patch`` as well.
    http_method_names = [
        'get', 'post', 'put', 'delete', 'head', 'patch', 'options', 'trace']

    #: A list of URL patterns which will be used by ``API.register`` to build
    #: the URLconf entries. The format is a list of tuples containing
    #: (regular expression, URL name suffix).
    urls = [
        (r'^$', 'list', {
            'request_type': 'list',
        }),
        (r'^(?P<pk>\d+)/$', 'detail', {
            'request_type': 'detail',
        }),
        (r'^(?P<pks>(?:\d+;)*\d+);?/$', 'set', {
            'request_type': 'set',
        }),
    ]

    def dispatch(self, request, *args, **kwargs):
        """
        This method is almost the same as Django's own
        ``generic.View.dispatch()``, but there are a few subtle differences:

        - It uses ``self.request``, ``self.args`` and ``self.kwargs`` in all
          places
        - It calls ``self.unserialize_request()`` after assigning the
          aforementioned variables on ``self`` which may modify all aspects and
          all variables (f.e.  deserialize a JSON request and serialize it
          again to look like a standard POST request) and only then determines
          whether the request should be handled by this view at all.
        - The return value of the ``get()``, ``post()`` etc. methods is passed
          to ``self.serialize_response()`` and only then returned to the
          client. The processing methods should return data (a ``dict``
          instance most of the time) which is then serialized into the
          requested format or some different supported format.
        """

        # The following three lines can be removed when we move to
        # Django 1.5 only
        self.request = request
        self.args = args
        self.kwargs = kwargs

        response = self.unserialize_request()
        if response:
            return response

        # Try to dispatch to the right method; if a method doesn't exist,
        # defer to the error handler. Also defer to the error handler if the
        # request method isn't on the approved list.
        method = self.request.method.lower()
        if method == 'head':
            method = 'get'
        handler = self.http_method_not_allowed

        if method in self.http_method_names:
            self.request_type = kwargs.get('request_type')
            if (self.request_type
                    and hasattr(self, '%s_%s' % (method, self.request_type))):
                handler = getattr(self, '%s_%s' % (method, self.request_type))
            elif hasattr(self, method):
                handler = getattr(self, method)

        try:
            return self.serialize_response(
                handler(self.request, *self.args, **self.kwargs))
        except Http404 as exc:
            return self.serialize_response(
                {'error': exc.args[0]},
                status=http_client.NOT_FOUND)
        except APIException as exc:
            logger.warning(
                'APIException: %s, %r' % (exc, exc.__dict__),
                extra={
                    'request': self.request,
                })

            data = {
                'error': exc.error_message,
            }
            data.update(exc.data)
            return self.serialize_response(data, status=exc.status)

    def unserialize_request(self):
        """
        This method standardizes various aspects of the incoming request, f.e.
        decoding of JSON requests etc.

        The "real" processing methods should not have to distinguish between
        varying request types anymore.

        If this method returns anything, it is treated as a response and
        short-circuits the resource processing.
        """
        return RequestParser().parse(self.request)

    def serialize_response(self, response, status=http_client.OK,
                           headers=None):
        """
        Serializes the response into an appropriate format for the wire such
        as JSON. ``HttpResponse`` instances are returned directly.
        """
        if isinstance(response, HttpResponse):
            return response

        return Serializer().serialize(
            response,
            request=self.request,
            status=status,
            output_format=self.request.GET.get('format'),
            headers=headers)

    def get_query_set(self):
        """
        Returns the queryset used by this resource. If you need access or
        visibility control, add it here.
        """
        if self.queryset:
            return self.queryset._clone()
        elif self.model:
            return self.model._default_manager.all()

    def apply_filters(self, queryset):
        """
        Applies filters and search queries to the queryset. This method will
        only be called for list views, not when the user requested sets or
        single instances.
        """
        return queryset

    def detail_object_or_404(self):
        """
        Returns the current object for detail resources such as
        ``/api/product/1/``.
        """
        return get_object_or_404(self.get_query_set(), pk=self.kwargs['pk'])

    def set_objects_or_404(self):
        """
        Returns the current set of objects for set resources such as
        ``/api/product/1;3/``.
        """
        pks = set(pk for pk in self.kwargs['pks'].split(';') if pk)
        set_ = self.get_query_set().in_bulk(pks).values()

        if len(pks) != len(set_):
            raise Http404('Some objects do not exist.')

        return set_

    def page_objects_or_404(self):
        """
        Returns the current page for list resources such as
        ``/api/product/?limit=20&offset=40``. Applies filtering using
        ``apply_filters`` as well.
        """
        queryset = self.apply_filters(self.get_query_set())

        try:
            offset = int(self.request.GET.get('offset'))
        except (TypeError, ValueError):
            offset = 0

        try:
            limit = int(self.request.GET.get('limit'))
        except (TypeError, ValueError):
            limit = self.limit_per_page

        # Do not allow more than max_limit_per_page entries in one request,
        # ever
        limit = min(limit, self.max_limit_per_page)

        # Sanitize range
        offset = max(offset, 0)
        limit = max(limit, 0)

        return Page(queryset[offset:offset + limit], offset, limit, queryset)

    def get_detail(self, request, *args, **kwargs):
        kw = {}
        if request.GET.get('full'):
            kw['inline_depth'] = 1
        return self.api.serialize_instance(
            self.detail_object_or_404(),
            build_absolute_uri=request.build_absolute_uri,
            **kw)

    def get_set(self, request, *args, **kwargs):
        return {
            'objects': [
                self.api.serialize_instance(
                    instance,
                    build_absolute_uri=request.build_absolute_uri,
                ) for instance in self.set_objects_or_404()
            ],
        }

    def get_list(self, request, *args, **kwargs):
        page = self.page_objects_or_404()

        list_url = api_reverse(
            page.full_queryset.model, 'list', api_name=self.api.name)

        meta = {
            'offset': page.offset,
            'limit': page.limit,
            'total': page.full_queryset.count(),
            'previous': None,
            'next': None,
        }

        if page.offset > 0:
            meta['previous'] = request.build_absolute_uri('%s?%s' % (
                list_url,
                querystring(
                    self.request.GET,
                    exclude=('offset', 'limit'),
                    offset=max(0, page.offset - page.limit),
                    limit=page.limit,
                ),
            ))

        if page.offset + page.limit < meta['total']:
            meta['next'] = request.build_absolute_uri('%s?%s' % (
                list_url,
                querystring(
                    self.request.GET,
                    exclude=('offset', 'limit'),
                    offset=page.offset + page.limit,
                    limit=page.limit,
                ),
            ))

        return {
            'objects': [
                self.api.serialize_instance(
                    instance,
                    build_absolute_uri=request.build_absolute_uri,
                ) for instance in page.queryset],
            'meta': meta,
        }

    def options(self, request, *args, **kwargs):
        # XXX This will be removed as soon as we switch to Django 1.5 only
        response = HttpResponse()
        response['Allow'] = ', '.join(self._allowed_methods())
        response['Content-Length'] = 0
        return response

    def _allowed_methods(self):
        methods = set(m.upper() for m in self.http_method_names if (
            hasattr(self, m)
            or hasattr(self, '%s_%s' % (m, self.request_type))
        ))
        if 'GET' in methods:
            methods.add('HEAD')
        return sorted(methods)



from __future__ import absolute_import, unicode_literals

import httplib

from django.contrib.messages.api import get_messages

from towel.api import Resource, APIException


class FrankenResource(Resource):
    """
    Really ugly and hacky way of reusing customizations made in a ``ModelView``
    subclass for API resources. Reuses the following aspects of the
    ``ModelView`` instance:

    - Basic queryset filtering, i.e. ``get_query_set``
    - Form handling and saving, i.e. ``get_form``, ``get_form_instance``,
      ``save_form``, ``save_model`` and ``post_save``
    - Permissions management, i.e. ``adding_allowed``, ``editing_allowed``,
      ``deletion_allowed``
    """

    #: ``ModelView`` instance used for providing permissions and write
    #: access to the exposed model
    modelview = None

    def get_query_set(self):
        return self.modelview.get_query_set(self.request)

    def post_list(self, request, *args, **kwargs):
        """
        POST handler. Only supports full creation of objects by posting to
        the listing endpoint currently.
        """
        if not self.modelview.adding_allowed(request):
            raise APIException(status=httplib.FORBIDDEN)

        form_class = self.modelview.get_form(
            request,
            change=False)
        form = self.modelview.get_form_instance(
            request,
            form_class=form_class,
            change=False)

        try:
            is_valid = form.is_valid()
        except TypeError as exc:
            # This can happen when POSTing something of type
            # application/json with a list instead of a single entry,
            # e.g. {"customer_id": ["1"]}
            raise APIException('Malformed data', data={
                'exception': '%s' % exc})

        if not is_valid:
            raise APIException(data={
                'validation': form.errors,
            })

        instance = self.modelview.save_form(request, form, change=False)
        self.modelview.save_model(request, instance, form, change=False)
        self.modelview.post_save(request, instance, form, {}, change=False)

        data = self.api.serialize_instance(
            instance,
            build_absolute_uri=request.build_absolute_uri)
        return self.serialize_response(
            data,
            status=httplib.CREATED,
            headers={'Location': data['__uri__']})

    def put_detail(self, request, *args, **kwargs):
        """
        PUT handler. Only supports update of existing resources. Sets are not
        supported.

        You are required to provide the full set of fields, otherwise
        validation fails. If you are looking for partial updates, have a look
        at PATCH.
        """
        instance = self.detail_object_or_404()

        if not self.modelview.editing_allowed(request, instance):
            raise APIException(status=httplib.FORBIDDEN)

        # The ModelView code only does the right thing when method is POST
        request.method = 'POST'

        form_class = self.modelview.get_form(
            request,
            instance=instance,
            change=True)
        form = self.modelview.get_form_instance(
            request,
            form_class=form_class,
            instance=instance,
            change=True)

        if not form.is_valid():
            raise APIException(data={
                'validation': form.errors,
            })

        instance = self.modelview.save_form(request, form, change=True)
        self.modelview.save_model(request, instance, form, change=True)
        self.modelview.post_save(request, instance, form, {}, change=True)

        data = self.api.serialize_instance(
            instance, build_absolute_uri=request.build_absolute_uri)
        return self.serialize_response(data, status=httplib.OK)

    def patch_detail(self, request, *args, **kwargs):
        """
        PATCH handler. Only supports update of existing resources.

        This handler offloads the work to the PUT handler. It starts with the
        serialized representation from the database, overwrites values using
        the data from the PATCH request and calls PUT afterwards.
        """
        instance = self.detail_object_or_404()

        if not self.modelview.editing_allowed(request, instance):
            raise APIException(status=httplib.FORBIDDEN)

        data = self.api.serialize_instance(
            instance, build_absolute_uri=request.build_absolute_uri)
        for key in request.POST:
            if isinstance(data[key], (list, tuple)):
                data[key] = request.POST.getlist(key)
            else:
                data[key] = request.POST[key]
        request.POST = data

        return self.put_detail(request, *args, **kwargs)

    def delete_detail(self, request, *args, **kwargs):
        """
        DELETE handler. Only supports deletion of single items at the moment.
        """
        instance = self.detail_object_or_404()

        if not self.modelview.deletion_allowed(request, instance):
            raise APIException(status=httplib.FORBIDDEN, data={
                'messages': [{
                    'message': '%s' % msg,
                    'tags': msg.tags,
                } for msg in get_messages(request)],
            })

        instance.delete()
        return self.serialize_response({}, status=httplib.NO_CONTENT)



from __future__ import absolute_import, unicode_literals

import json

from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse
from django.utils.cache import patch_vary_headers
from django.utils.six.moves import http_client

from towel.api.mimeparse import best_match


class Serializer(object):
    """
    API response serializer

    Handles content type negotiation using the HTTP Accept header if the
    format isn't overridden.
    """
    def serialize(self, data, output_format=None, request=None,
                  status=http_client.OK, headers=None):
        """
        Returns a ``HttpResponse`` containing the serialized response in the
        format specified explicitly in ``output_format`` (either as a MIME
        type or as a simple identifier) or according to the HTTP Accept
        header specified in the passed request instance. The default status
        code is ``200 OK``, if that does not fit you'll have to specify a
        different code yourself.

        Returns a ``406 Not acceptable`` response if the requested format is
        unknown or unsupported. Currently, the following formats are
        supported:

        - ``json`` or ``application/json``

        Usage::

            return Serializer().serialize(
                {'response': 'Hello world'},
                output_format='application/json',
                status=httplib.OK)

        or::

            return Serializer().serialize(
                {'response': 'Hello world'},
                request=request,
                status=httplib.OK)
        """
        if output_format is None and request is None:
            raise TypeError(
                'Provide at least one of output_format and request.')

        if output_format is None:
            # Thanks django-tastypie!
            try:
                output_format = best_match(
                    reversed([
                        'application/json',
                    ]),
                    request.META.get('HTTP_ACCEPT', ''),
                )
            except (IndexError, ValueError):
                pass

        if output_format in ('application/json', 'json'):
            response = self.to_json(data)

        else:
            # Cannot raise ClientError here because the APIException handler
            # calls into this method too.
            response = HttpResponse('Not acceptable')
            status = http_client.NOT_ACCEPTABLE

        if headers:
            for key, value in headers.items():
                response[key] = value

        patch_vary_headers(response, ('Accept',))
        response.status_code = status
        return response

    def to_json(self, data):
        return HttpResponse(
            json.dumps(data, cls=DjangoJSONEncoder),
            content_type='application/json',
        )



from __future__ import absolute_import, unicode_literals

from django import forms, template
from django.template.loader import render_to_string


register = template.Library()


def _type_class(item):
    if isinstance(item.field.widget, forms.CheckboxInput):
        return 'checkbox'
    elif isinstance(item.field.widget, forms.DateInput):
        return 'date'
    elif isinstance(item.field.widget, forms.HiddenInput):
        return 'hidden'
    elif isinstance(
            item.field.widget,
            (forms.RadioSelect, forms.CheckboxSelectMultiple)):
        return 'list'
    elif isinstance(item.field.widget, forms.Select):
        return 'choice'
    return 'default'


@register.simple_tag
def form_items(form):
    """
    Render all form items::

        {% form_items form %}
    """
    return ''.join(render_to_string('towel/_form_item.html', {
        'item': field,
        'is_checkbox': isinstance(field.field.widget, forms.CheckboxInput),
        'type_class': _type_class(field),
    }) for field in form if field.name != 'ignore_warnings')


@register.inclusion_tag('towel/_form_item.html')
def form_item(item, additional_classes=None):
    """
    Helper for easy displaying of form items:

    ::

        {% for field in form %}
            {% form_item field %}
        {% endfor %}
    """

    return {
        'item': item,
        'additional_classes': additional_classes,
        'is_checkbox': isinstance(item.field.widget, forms.CheckboxInput),
        'type_class': _type_class(item),
    }


@register.inclusion_tag('towel/_form_item_plain.html')
def form_item_plain(item, additional_classes=None):
    """
    Helper for easy displaying of form items without any additional
    tags (table cells or paragraphs) or labels::

        {% form_item_plain field %}
    """

    return {
        'item': item,
        'additional_classes': additional_classes,
        'is_checkbox': isinstance(item.field.widget, forms.CheckboxInput),
        'type_class': _type_class(item),
    }


@register.tag
def form_errors(parser, token):
    """
    Show all form and formset errors::

        {% form_errors form formset1 formset2 %}

    Silently ignores non-existant variables.
    """

    tokens = token.split_contents()

    return FormErrorsNode(*tokens[1:])


class FormErrorsNode(template.Node):
    def __init__(self, *items):
        self.items = [template.Variable(item) for item in items]

    def render(self, context):
        items = []
        for item in self.items:
            try:
                var = item.resolve(context)
                if isinstance(var, dict):
                    items.extend(var.values())
                elif isinstance(var, (list, tuple)):
                    items.extend(var)
                else:
                    items.append(var)
            except template.VariableDoesNotExist:
                # We do not care too much
                pass

        errors = False
        has_non_field_errors = False

        form_list = []
        formset_list = []

        for i in items:
            if isinstance(i, forms.BaseForm):
                form_list.append(i)
            else:
                formset_list.append(i)

            if getattr(i, 'non_field_errors', lambda: None)():
                errors = True
                has_non_field_errors = True
            if getattr(i, 'errors', None):
                errors = True

        if not errors:
            return ''

        return render_to_string('towel/_form_errors.html', {
            'forms': form_list,
            'formsets': formset_list,
            'errors': errors,
            'has_non_field_errors': has_non_field_errors,
        })


@register.tag
def form_warnings(parser, token):
    """
    Show all form and formset warnings::

        {% form_warnings form formset1 formset2 %}

    Silently ignores non-existant variables.
    """

    tokens = token.split_contents()

    return FormWarningsNode(*tokens[1:])


class FormWarningsNode(template.Node):
    def __init__(self, *items):
        self.items = [template.Variable(item) for item in items]

    def render(self, context):
        items = []
        for item in self.items:
            try:
                var = item.resolve(context)
                if isinstance(var, dict):
                    items.extend(var.values())
                elif isinstance(var, (list, tuple)):
                    items.extend(var)
                else:
                    items.append(var)
            except template.VariableDoesNotExist:
                # We do not care too much
                pass

        warnings = False

        form_list = []
        formset_list = []

        for i in items:
            if isinstance(i, forms.BaseForm):
                form_list.append(i)
                if getattr(i, 'warnings', None):
                    warnings = True
            else:
                formset_list.append(i)
                if any(getattr(f, 'warnings', None) for f in i):
                    warnings = True

        if not warnings:
            return ''

        return render_to_string('towel/_form_warnings.html', {
            'forms': form_list,
            'formsets': formset_list,
            'warnings': True,
        })


@register.tag
def dynamic_formset(parser, token):
    """
    Implements formsets where subforms can be added using the
    ``towel_add_subform`` javascript method::

        {% dynamic_formset formset "activities" %}
            ... form code
        {% enddynamic_formset %}
    """

    tokens = token.split_contents()
    nodelist = parser.parse(('enddynamic_formset',))
    parser.delete_first_token()

    return DynamicFormsetNode(tokens[1], tokens[2], nodelist)


class DynamicFormsetNode(template.Node):
    def __init__(self, formset, slug, nodelist):
        self.formset = template.Variable(formset)
        self.slug = template.Variable(slug)
        self.nodelist = nodelist

    def render(self, context):
        formset = self.formset.resolve(context)
        slug = self.slug.resolve(context)

        result = []

        context.update({
            'empty': True,
            'form_id': '%s-empty' % slug,
            'form': formset.empty_form,
        })
        result.append('<script type="text/template" id="%s-empty">' % slug)
        result.append(self.nodelist.render(context))
        result.append('</script>')
        context.pop()

        for idx, form in enumerate(formset.forms):
            context.update({
                'empty': False,
                'form_id': '%s-%s' % (slug, idx),
                'form': form,
            })
            result.append(self.nodelist.render(context))
            context.pop()

        return ''.join(result)



from __future__ import absolute_import, unicode_literals

import json

from django.forms.models import model_to_dict
from django.http import Http404, HttpResponse
from django.shortcuts import get_object_or_404, render

from towel.modelview import ModelView
from towel.utils import app_model_label, changed_regions


class EditLiveModelView(ModelView):
    #: The form class used for live editing. Should only contain fields
    #: for which editing through the editlive mechanism is allowed.
    editlive_form = None

    def editlive(self, request, *args, **kwargs):
        if not self.editlive_form:
            raise Http404('No live editing support.')

        instance = self.get_object_or_404(request, *args, **kwargs)

        data = model_to_dict(
            instance,
            fields=self.editlive_form._meta.fields,
            exclude=self.editlive_form._meta.exclude)

        for key, value in request.POST.items():
            data[key] = value

        form = self.editlive_form(data, instance=instance, request=request)

        if form.is_valid():
            return self.response_editlive(request, form.save(), form, {})

        return HttpResponse(
            json.dumps({'!form-errors': dict(form.errors)}),
            content_type='application/json')

    def response_editlive(self, request, new_instance, form, formsets):
        regions = {}
        self.render_detail(request, {
            self.template_object_name: new_instance,
            'regions': regions,
        })
        data = {'!form-errors': {}}
        data.update(changed_regions(regions, form.changed_data))
        return HttpResponse(json.dumps(data), content_type='application/json')


class ParentModelView(EditLiveModelView):
    def response_edit(self, request, new_instance, form, formsets):
        return self.response_editlive(request, new_instance, form, formsets)

    def render_form(self, request, context, change):
        if change:
            context.setdefault('base_template', 'modal.html')
        return super(ParentModelView, self).render_form(
            request, context, change=change)


class InlineModelView(EditLiveModelView):
    parent_attr = 'parent'

    @property
    def parent_class(self):
        return self.model._meta.get_field(self.parent_attr).rel.to

    def get_object(self, request, *args, **kwargs):
        if 'pk' in kwargs:
            kwargs[self.parent_attr] = kwargs.pop('parent')
        return super(InlineModelView, self).get_object(
            request, *args, **kwargs)

    def add_view(self, request, parent):
        request._parent = get_object_or_404(self.parent_class, id=parent)
        return super(InlineModelView, self).add_view(request)

    def save_model(self, request, instance, form, change):
        if hasattr(request, '_parent'):
            setattr(instance, self.parent_attr, request._parent)
        super(InlineModelView, self).save_model(
            request, instance, form=form, change=change)

    def response_add(self, request, instance, *args, **kwargs):
        regions = {}
        render(
            request,
            '%s/%s_detail.html' % app_model_label(self.parent_class),
            {
                'object': getattr(instance, self.parent_attr),
                'regions': regions,
            },
        )
        return HttpResponse(
            json.dumps(changed_regions(regions, [
                '%s_set' % self.model.__name__.lower(),
            ])),
            content_type='application/json',
        )

    response_delete = response_editlive = response_edit = response_add
    # TODO what about response_adding_denied, response_editing_denied and
    # response_deletion_denied?



from __future__ import absolute_import, unicode_literals

from django.core.urlresolvers import NoReverseMatch, reverse
from django.conf.urls import patterns, include, url
from django.db import models
from django.db.models.related import RelatedObject
from django.http import HttpResponse
from django.utils.encoding import force_text
from django.utils.functional import curry
from django.utils.six.moves import http_client
from django.views.decorators.csrf import csrf_exempt

from towel.utils import app_model_label
from .base import api_reverse
from .resources import Resource
from .serializers import Serializer


class API(object):
    """
    This is the main API object. It does not do much except give an overview
    over all resources. It will hold the necessary bits to have more than one
    API with the same models or resources at the same time (f.e. versions).

    Usage::

        # ... other imports ...
        from functools import partial
        from towel.api import API, serialize_model_instance

        api_v1 = API('v1')

        # Customize serialization: Never include phone numbers and email
        # addresses of customers in API output.
        api_v1.register(
            Customer,
            serializer=partial(serialize_model_instance,
                exclude=('phone', 'email')),
            view_init={
                'queryset': Customer.objects.filter(is_active=True),
                })

        api_v1.register(
            Product,
            view_init={
                'queryset': Product.objects.filter(is_active=True)
                })

        api_v1.register(
            Product,
            canonical=False,
            prefix=r'^library/',
            view_class=LibraryResource,
            view_init={
                'queryset': Product.objects.filter(is_active=True),
                })

        urlpatterns = patterns('',
            url(r'^v1/', include(api_v1.urls)),
        )

    With authentication::

        api_v2 = API('v2', decorators=[csrf_exempt, login_required])

        # register resources as before
    """

    def __init__(self, name, decorators=[csrf_exempt]):
        self.name = name
        self.decorators = decorators

        self.resources = []
        self.serializers = {}
        self.views = []

        self.default_serializer = serialize_model_instance

    @property
    def urls(self):
        """
        Inclusion point in your own URLconf

        Usage::

            from .views import api_v1

            urlpatterns = patterns('',
                url(r'^api/v1/', include(api_v1.urls)),
            )
        """

        def view(request):
            return self.root(request)
        for dec in reversed(self.decorators):
            view = dec(view)

        urlpatterns = [
            url(r'^$', view, name='api_%s' % self.name),
        ]

        for view in self.views:
            urlpatterns.append(url(view['prefix'], view['view']))

        for resource in self.resources:
            urlpatterns.append(url(
                resource['prefix'],
                include(resource['urlpatterns']),
            ))

        return patterns('', *urlpatterns)

    def root(self, request):
        """
        Main API view, returns a list of all available resources
        """
        if request.method == 'OPTIONS':
            response = HttpResponse()
            response['Allow'] = 'GET, HEAD, OPTIONS'
            response['Content-Length'] = 0
            return response

        elif request.method not in ('GET', 'HEAD'):
            return Serializer().serialize(
                {
                    'error': 'Not acceptable',
                },
                request=request,
                status=http_client.METHOD_NOT_ALLOWED,
                output_format=request.GET.get('format'),
            )

        response = {
            '__str__': self.name,
            '__uri__': request.build_absolute_uri(
                reverse('api_%s' % self.name)),
            'resources': [],
        }

        for view in self.views:
            response.setdefault('views', []).append({
                '__str__': view['prefix'].strip('^').strip('/'),
                '__uri__': request.build_absolute_uri(''.join((
                    response['__uri__'],
                    view['prefix'].strip('^')))),
            })

        for resource in self.resources:
            r = {
                '__str__': resource['model'].__name__.lower(),
                '__uri__': request.build_absolute_uri(''.join((
                    response['__uri__'],
                    resource['prefix'].strip('^')))),
            }

            response['resources'].append(r)
            if resource['canonical']:
                response[resource['model'].__name__.lower()] = r

        return Serializer().serialize(
            response,
            request=request,
            output_format=request.GET.get('format'))

    def register(self, model, view_class=None, canonical=True,
                 decorators=None, prefix=None, view_init=None,
                 serializer=None):
        """
        Registers another resource on this API. The sole required argument is
        the Django model which should be exposed. The other arguments are:

        - ``view_class``: The resource view class used, defaults to
          :class:`towel.api.Resource`.
        - ``canonical``: Whether this resource is the canonical location of
          the model in this API. Allows registering the same model several
          times in the API (only one location should be the canonical
          location!)
        - ``decorators``: A list of decorators which should be applied to the
          view. Function decorators only, method decorators aren't supported.
          The list is applied in reverse, the order is therefore the same as
          with the ``@`` notation. If unset, the set of decorators is
          determined from the API initialization. Pass an empty list if you
          want no decorators at all. Otherwise API POSTing will have to
          include a valid CSRF middleware token.
        - ``prefix``: The prefix for this model, defaults to the model name in
          lowercase. You should include a caret and a trailing slash if you
          specify this yourself (``prefix=r'^library/'``).
        - ``view_init``: Python dictionary which contains keyword arguments
          used during the instantiation of the ``view_class``.
        - ``serializer``: Function which takes a model instance, the API
          instance and additional keyword arguments (accept ``**kwargs`` for
          forward compatibility) and returns the serialized representation as
          a Python dict.
        """

        view_class = view_class or Resource
        view_init = view_init or {}

        if 'model' not in view_init:
            if 'queryset' in view_init:
                view_init['model'] = view_init['queryset'].model
            else:
                view_init['model'] = model

        view = view_class.as_view(api=self, **view_init)

        name = lambda ident: None
        if canonical:
            name = lambda ident: '_'.join(
                (self.name,) + app_model_label(model) + (ident,))

        if decorators is None:
            decorators = self.decorators

        if decorators:
            for dec in reversed(decorators):
                view = dec(view)

        self.resources.append({
            'model': model,
            'canonical': canonical,
            'prefix': prefix or r'^%s/' % model.__name__.lower(),
            'urlpatterns': patterns('', *[
                url(regex, view, data, name=name(suffix))
                for regex, suffix, data in view_class.urls
            ]),
        })

        if serializer:
            self.serializers[model] = serializer

    def set_default_serializer(self, serializer):
        """
        By default, ``serialize_model_instance`` is used to serialize models.
        This can be changed by passing a different function to this method.
        """
        self.default_serializer = serializer

    def serialize_instance(self, instance, **kwargs):
        """
        Returns a serialized version of the passed model instance

        This method should always be used for serialization, because it knows
        about custom serializers specified when registering resources with
        this API.
        """
        serializer = self.serializers.get(
            instance.__class__,
            self.default_serializer)
        return serializer(instance, api=self, **kwargs)

    def add_view(self, view, prefix=None, decorators=None):
        """
        Add custom views to this API

        The prefix is automatically determined if not given based on the
        function name.

        The view receives an additional keyword argument ``api`` containing
        the API instance.
        """

        prefix = prefix or r'^%s/' % view.__name__

        if decorators is None:
            decorators = self.decorators

        view = curry(view, api=self)
        for dec in reversed(decorators):
            view = dec(view)

        self.views.append({
            'prefix': prefix,
            'view': view,
        })


def serialize_model_instance(instance, api, inline_depth=0,
                             fields=(), exclude=(), only_registered=True,
                             build_absolute_uri=lambda uri: uri,
                             **kwargs):
    """
    Serializes a single model instance.

    If ``inline_depth`` is a positive number, ``inline_depth`` levels of
    related objects are inlined. The performance implications of this feature
    might be severe! Note: Additional arguments specified when calling
    ``serialize_model_instance`` such as ``exclude``, ``only_registered`` and
    further keyword arguments are currently **not** forwarded to inlined
    objects. Those parameters should be set upon resource registration time as
    documented in the ``API`` docstring above.

    The ``fields`` and ``exclude`` parameters are especially helpful when used
    together with ``functools.partial``.

    Set ``only_registered=False`` if you want to serialize models which do not
    have a canonical URI inside this API.

    ``build_absolute_uri`` should be a callable which transforms any passed
    URI fragment into an absolute URI including the protocol and the hostname,
    for example ``request.build_absolute_uri``.

    This implementation has a few characteristics you should be aware of:

    - Only objects which have a canonical URI inside this particular API are
      serialized; if no such URI exists, this method returns ``None``. This
      behavior can be overridden by passing ``only_registered=False``.
    - Many to many relations are only processed if ``inline_depth`` has a
      positive value. The reason for this design decision is that the database
      has to be queried for showing the URIs of related objects anyway and
      because of that we either show the full objects or nothing at all.
    - Some fields (currently only fields with choices) have a machine readable
      and a prettified value. The prettified values are delivered inside the
      ``__pretty__`` dictionary for your convenience.
    - The primary key of the model instance is always available as
      ``__pk__``.
    """

    # It's not exactly a fatal error, but it helps during development. This
    # statement will disappear in the future.
    assert not kwargs, 'Unknown keyword arguments to serialize_model_instance'

    uri = api_reverse(
        instance, 'detail', api_name=api.name, pk=instance.pk,
        fail_silently=True)

    if uri is None and only_registered:
        return None

    data = {
        '__uri__': build_absolute_uri(uri),
        '__str__': force_text(instance),
        '__pretty__': {},
        '__pk__': instance.pk,
    }
    opts = instance._meta

    for f_name in opts.get_all_field_names():
        f, model, direct, m2m = opts.get_field_by_name(f_name)

        if fields and f.name not in fields:
            continue

        if f.name in exclude:
            continue

        # TODO maybe check before querying the database whether the objects
        # are included in the API or only_registered=False?

        if isinstance(f, (models.ManyToManyField, RelatedObject)):
            if inline_depth > 0:
                is_relobj = isinstance(f, RelatedObject)
                name = f.get_accessor_name() if is_relobj else f.name

                if is_relobj and not f.field.rel.multiple:
                    try:
                        obj = getattr(instance, name)
                    except models.ObjectDoesNotExist:
                        obj = None

                    data[name] = api.serialize_instance(
                        obj,
                        inline_depth=inline_depth - 1,
                        build_absolute_uri=build_absolute_uri,
                        only_registered=only_registered,
                    ) if obj else None
                else:
                    related = [api.serialize_instance(
                        obj,
                        inline_depth=inline_depth - 1,
                        build_absolute_uri=build_absolute_uri,
                        only_registered=only_registered,
                    ) for obj in getattr(instance, name).all()]
                    if any(related):
                        data[name] = related

        elif f.rel:
            value = f.value_from_object(instance)
            if value is None:
                data[f.name] = None
                continue

            try:
                data[f.name] = build_absolute_uri(api_reverse(
                    f.rel.to,
                    'detail',
                    api_name=api.name,
                    pk=f.value_from_object(instance)))
            except NoReverseMatch:
                if only_registered:
                    continue

            if inline_depth > 0:
                related = getattr(instance, f.name)

                if related:
                    # XXX What about only_registered, kwargs? Should they be
                    # passed to other calls as well, or should we assume that
                    # customization can only happen using functools.partial
                    # upon registration time?
                    data[f.name] = api.serialize_instance(
                        related,
                        inline_depth=inline_depth - 1,
                        build_absolute_uri=build_absolute_uri,
                        only_registered=only_registered,
                    )

        elif isinstance(f, models.FileField):
            try:
                value = f.value_from_object(instance)
                data[f.name] = build_absolute_uri(value.url)
            except ValueError:
                data[f.name] = ''

        else:
            data[f.name] = f.value_from_object(instance)

            if f.flatchoices:
                data['__pretty__'][f.name] = force_text(
                    dict(f.flatchoices).get(data[f.name], '-'))

    return data
